// Date.h
// cs7s14
#include <iostream>
#include <string>
#include <cassert>

using namespace std;
//using std::cout;
//using std::cin;
//using std::endl;
//using std::istream;
//using std::ostream;

const int FIRST_YEAR = 1701;
const int LAST_YEAR = 10000;

//enum class Month {January = 1, February, March, April, May, June,
//            July, August, September, October, November, December};

//enum Era {BC, AD};

class Date {
    friend istream& operator>>(istream&, Date&);
    friend ostream& operator<<(ostream&, const Date&);
    friend bool operator==(const Date&, const Date&);
    friend bool operator!=(const Date&, const Date&);
    friend bool operator<(const Date&, const Date&);
    friend bool operator>(const Date&, const Date&);
    friend bool operator<=(const Date&, const Date&);
    friend bool operator>=(const Date&, const Date&);
    friend Date operator+(const Date&, const int);
    friend Date operator-(const Date&, const int);
    friend int operator-(const Date&, const Date&);
public:
    //Date(int m, int d, int y, bool e) : _month(m), _day(d), _year(y), _era(e) {}
    //Date() {}  //had to comment this out because of ambiguous call to constructor
    ~Date() {};
    //void print();
    Date(int=1702, int=1, int=1);
    int day_of_year() const;
    int year() const;
    int month() const;
    int day() const;
    int era() const;
    int days() const;
    string weekday() const;
    Date operator++();
    Date operator--();
    Date& operator+=(const int);
    Date& operator-=(const int);
    void enterDiary();
    string showDiary() const;

private:
    int _day; //days since FIRST_YEAR
    string _diary;
};

void Date::enterDiary()
{
    //string entry;
    cout << "\nEnter events for " << this->weekday() << ", " << *this << ": ";
    getline(cin, _diary);
}

string Date::showDiary() const {
    cout << this->_diary << endl;
}

bool isLeap(int y) {
    if(y%400 == 0) return true;
    if(y%100 == 0) return false;
    if(y%4 == 0) return true;
    return false;
}

int days_in_month(int m, int y) {
    if (m == 2) return (isLeap(y) ? 29 : 28);
    if (m == 4 || m == 6 || m == 9 || m == 11) return 30;
    return 31;
}

int elapsed(int y, int m, int d)
// returns the number of days between 0-1-1 and y-m-d
{
    int n = 0;
    for (int yy=FIRST_YEAR; yy<y; yy++)
        n += (isLeap(yy) ? 366 : 365);
    for (int mm=1; mm<m; mm++)
        n += days_in_month(mm, y);
    return n + d;
}



istream& operator>>(istream& istr, Date& x)
{ int year, month, day;
  char c;  // to eat the '-'
  cin >> year >> c >> month >> c >> day;
  x._day = elapsed(year, month, day);
  return istr;
}

ostream& operator<<(ostream& ostr, const Date& x) {
    switch(x.month()) {
        case 1 : ostr << "January"; break;
        case 2 : ostr << "February"; break;
        case 3 : ostr << "March"; break;
        case 4 : ostr << "April"; break;
        case 5 : ostr << "May"; break;
        case 6 : ostr << "June"; break;
        case 7 : ostr << "July"; break;
        case 8 : ostr << "August"; break;
        case 9 : ostr << "September"; break;
        case 10 : ostr << "October"; break;
        case 11 : ostr << "November"; break;
        case 12 : ostr << "December"; break;
    }
    ostr << " ";
    ostr << x.day() << ", ";
    ostr << x.year();;// << " " << x.era();
    return ostr;
}

inline bool operator==(const Date& x, const Date& y) {
    return bool(x._day == y._day);
}

inline bool operator!=(const Date& x, const Date& y) {
    return bool(x._day != y._day);
}

inline bool operator<(const Date& x, const Date& y) {
    return bool(x._day < y._day);
}

inline bool operator<=(const Date& x, const Date& y) {
    return bool(x._day <= y._day);
}

inline bool operator>(const Date& x, const Date& y) {
    return bool(x._day > y._day);
}

inline bool operator>=(const Date& x, const Date& y) {
    return bool(x._day >= y._day);
}

inline Date operator+(const Date& x, const int n) {
    Date y = x;
    y._day += n;
    return y;
}

inline Date operator-(const Date& x, const int n) {
    Date y = x;
    y._day -= n;
    return y;
}

inline int operator-(const Date& x, const Date& y) {
    return x._day - y._day;
}

Date::Date(int y, int m, int d) {
    assert(y >= FIRST_YEAR);
    assert(1 <= m && m <= 12);
    assert(1 <= d && d <= days_in_month(m, y));
    _day = elapsed(y, m, d);
}

int Date::day_of_year() const {
    int diy;  // number of days in year
    int n = _day;
    for (int y=FIRST_YEAR; y<LAST_YEAR; y++) {
        diy = (isLeap(y) ? 366 : 365);
        if (n <= diy) return n;
        n -= diy;
    }
    return n;
}

int Date::year() const {
    int diy;  // number of days in year
    int n = _day;
    for (int y=FIRST_YEAR; y<LAST_YEAR; y++) {
        diy = (isLeap(y) ? 366 : 365);
        if (n <= diy) return y;
        n -= diy;
    }
    return LAST_YEAR;
}

int Date::month() const {
    int dim;  // number of days in month
    int y = year();
    int n = day_of_year();
    for (int m=1; m<12; m++)   {
        dim = days_in_month(m, y);
        if (n <= dim) return m;
        n -= dim;
    }
    return 12;
}

int Date::day() const { // day of the month
    int dim;  // number of days in month
    int y = year();
    int n = day_of_year();
    for (int m=1; m<12; m++) {
        dim = days_in_month(m, y);
        if (n <= dim) return n;
        n -= dim;
    }
    return n;
}

inline int Date::days() const {
    return _day;
}

string Date::weekday() const
// Zeller's Algorithm (see Reilly & Federighi, p.309)
{
    const string WEEKDAY[7] = { "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" };
    int anchorDay, sum;
    int y = year();
    int m = month();
    int d = days();
    int centYr = y%100;
    int centYrby12 = centYr/12;
    int diff = centYr - 12*centYrby12;
    if (y < 1700) return "out of date";
    else if( y - 1800 < 100) anchorDay = 5; //WEEKDAY[5];
    else if (y - 1900 < 100) anchorDay = 3; //WEEKDAY[3];
    else if (y - 2000 < 100) anchorDay = 2; // WEEKDAY[2];
    else if (y - 2100 < 100) anchorDay = 0; //WEEKDAY[0];

    else if( y - 2200 < 100) anchorDay = 5; //WEEKDAY[5];
    else if (y - 2300 < 100) anchorDay = 3; //WEEKDAY[3];
    else if (y - 2400 < 100) anchorDay = 2; //WEEKDAY[2];
    else if (y - 2500 < 100) anchorDay = 0; //WEEKDAY[0];

    else if( y - 2600 < 100) anchorDay = 5; //WEEKDAY[5];
    else if (y - 2700 < 100) anchorDay = 3; //WEEKDAY[3];
    else if (y - 2800 < 100) anchorDay = 2; //WEEKDAY[2];
    else if (y - 2900 < 100) anchorDay = 0; //WEEKDAY[0];

    else if( y - 3000 < 100) anchorDay = 5; //WEEKDAY[5];
    else if (y - 3100 < 100) anchorDay = 3; //WEEKDAY[3];
    else if (y - 3200 < 100) anchorDay = 2; //WEEKDAY[2];
    else if (y - 3300 < 100) anchorDay = 0; //WEEKDAY[0];

    else if( y - 3400 < 100) anchorDay = 5; //WEEKDAY[5];
    else if (y - 3500 < 100) anchorDay = 3; //WEEKDAY[3];
    else if (y - 3600 < 100) anchorDay = 2; //WEEKDAY[2];
    else if (y - 3700 < 100) anchorDay = 0; //WEEKDAY[0];
    int divby4 = diff/4;
    sum = centYrby12 + diff + divby4 + anchorDay;
    int doomsday = sum%7;
    cout << "\nDoomsday for your year is " << WEEKDAY[doomsday] << endl;
//    m = 1 + (m + 9)%12;
//    if (m > 10) --y;
//    int c = y/100;
//    y %= 100;
//    int i = ((13*m - 1)/5 + d + y + y/4 + c/4 + 5*c)%7;
    return WEEKDAY[sum];
}


inline Date Date::operator++() {
    ++_day;
    return *this;
}

inline Date Date::operator--() {
    --_day;
    return *this;
}

inline Date& Date::operator+=(const int n) {
    _day += n;
    return *this;
}

inline Date& Date::operator-=(const int n) {
    _day -= n;
    return *this;
}


